/*S88 occupancy sensor interface to Command Station
Freely distributable for private, non commercial, use.

Connections for S88N bus:
S88N pin 1 +5V       to ARD VCC
S88N pin 2 DATA OUT  to ARD pin 1 = dataOut to central unit (next Ard in s88 chain)
S88N pin 3 GND       to ARD GND
S88N pin 4 CLOCK     to ARD pin 2, interrupt 0
S88N pin 5 GND       to ARD GND
S88N pin 6 LOAD/PS   to ARD pin 3, interrupt 1
S88N pin 2 DATA IN   to ARD pin 0 = dataIn from previous Ard in the S88 chain

Connections for sensors: see table in void Setup().
REMARK: inputs have the internal pullup resistor active, the sensors must pull the input to GND.
*/

const byte DATA_IN=0;      //data input from previous Arduino in S88 chain
const byte DATA_OUT=1;     //data output pin to central unit
//  sensors               1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15,16
const int INPUT_PIN[] = {12,13,14,15,16,18,17,19, 4, 5, 6, 7, 8, 9,11,10};
int clockCounter=0;
unsigned int sensors=0;
unsigned int stateRead=0;
unsigned long start_mil[17]; // value millis for timming
unsigned int data=0xffff;
//long loopCounter=0;      //used in lines 46 and 88, see there for explanation
//#define DEBUG 1

void setup() {
  pinMode(2, INPUT_PULLUP);
  attachInterrupt(0,clockInt,RISING); //pin 2 = clock interrupt
  pinMode(3, INPUT_PULLUP);
  attachInterrupt(1,loadPs,RISING);    //pin 3 = loadPs interrupt
  pinMode(DATA_IN,INPUT_PULLUP); //pin 0 = data in from previous Arduino S88 in chain
  pinMode(DATA_OUT, OUTPUT);     //pin 1 = data out to central or to next Arduino in S88 chain
  digitalWrite(DATA_OUT, LOW);
  memset (start_mil,0,17);
  for (int i = 0; i < 16; i++) {
    pinMode(INPUT_PIN[i], INPUT_PULLUP);
  }
  #ifdef DEBUG
    Serial.begin(9600);  // Pin 0 and 1 is not available! Used for test purposes only
  #endif
}

void loop() {
  //if (loopCounter==20){bitSet(sensors,0);}
  /*
  For an unknown reason the ECoS sets the first 8 bits to 1 after startup / reset of the S88 Arduino's.
  When one of the sensor inputs is changed, from there on everything goes well.
  Therefore, over here we give sensor bit 0 an automatic change after 1 second.
  The 1 second is created via 'loopCounter', which increments in the loadPs interrupt (line 88).
  There are appr0ximately 20 PS pulses per second, therefore we use 20 in the if statement.
  */
  for (int i = 0; i < 16; i++) {
    if (!digitalRead(INPUT_PIN[i])){
      bitSet(sensors,i);
    } else {
      bitClear(sensors,i);
    }      
  }
  //hold state
  for (int i = 0; i < 16; i++) {
    if (bitRead(sensors,i)==1) {
      bitSet(stateRead,i);
      start_mil[i] = millis();
    }
    if (start_mil[i] + 500 < millis()) {
       bitClear(stateRead,i);
    }
  }
  #ifdef DEBUG 
    if (start_mil[16] + 2000 < millis()) {
      Serial.print(loopCounter); // Pin 0 and 1 is not available! Used for test purposes only
      Serial.print(" - ");
      Serial.print(sensors,BIN);
      Serial.print(" - ");
      Serial.println(stateRead,BIN);
      start_mil[16] = millis();
    }
  #endif  
}

void loadPs() {
  clockCounter=0;
  data=stateRead;
  sensors=0;
  //loopCounter++; //Increment loopCounter to cretae a timer. See line 42 for explanation.
}

void clockInt() {
  digitalWrite(DATA_OUT,bitRead(data,clockCounter));
  delayMicroseconds(16); //Delay makes reading output signal from previous Arduino in chain
  bitWrite(data,clockCounter,digitalRead(DATA_IN));
  clockCounter = (clockCounter +1) % 16; 
}
